# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

import google.iam.v1.policy_pb2 as policy_pb2  # type: ignore
import google.protobuf.timestamp_pb2 as timestamp_pb2  # type: ignore
import google.rpc.status_pb2 as status_pb2  # type: ignore
import google.type.date_pb2 as date_pb2  # type: ignore
import proto  # type: ignore

from google.cloud.policysimulator_v1.types import explanations

__protobuf__ = proto.module(
    package="google.cloud.policysimulator.v1",
    manifest={
        "Replay",
        "ReplayResult",
        "CreateReplayRequest",
        "ReplayOperationMetadata",
        "GetReplayRequest",
        "ListReplayResultsRequest",
        "ListReplayResultsResponse",
        "ReplayConfig",
        "ReplayDiff",
        "AccessStateDiff",
        "ExplainedAccess",
    },
)


class Replay(proto.Message):
    r"""A resource describing a ``Replay``, or simulation.

    Attributes:
        name (str):
            Output only. The resource name of the ``Replay``, which has
            the following format:

            ``{projects|folders|organizations}/{resource-id}/locations/global/replays/{replay-id}``,
            where ``{resource-id}`` is the ID of the project, folder, or
            organization that owns the Replay.

            Example:
            ``projects/my-example-project/locations/global/replays/506a5f7f-38ce-4d7d-8e03-479ce1833c36``
        state (google.cloud.policysimulator_v1.types.Replay.State):
            Output only. The current state of the ``Replay``.
        config (google.cloud.policysimulator_v1.types.ReplayConfig):
            Required. The configuration used for the ``Replay``.
        results_summary (google.cloud.policysimulator_v1.types.Replay.ResultsSummary):
            Output only. Summary statistics about the
            replayed log entries.
    """

    class State(proto.Enum):
        r"""The current state of the
        [Replay][google.cloud.policysimulator.v1.Replay].

        Values:
            STATE_UNSPECIFIED (0):
                Default value. This value is unused.
            PENDING (1):
                The ``Replay`` has not started yet.
            RUNNING (2):
                The ``Replay`` is currently running.
            SUCCEEDED (3):
                The ``Replay`` has successfully completed.
            FAILED (4):
                The ``Replay`` has finished with an error.
        """

        STATE_UNSPECIFIED = 0
        PENDING = 1
        RUNNING = 2
        SUCCEEDED = 3
        FAILED = 4

    class ResultsSummary(proto.Message):
        r"""Summary statistics about the replayed log entries.

        Attributes:
            log_count (int):
                The total number of log entries replayed.
            unchanged_count (int):
                The number of replayed log entries with no
                difference between baseline and simulated
                policies.
            difference_count (int):
                The number of replayed log entries with a
                difference between baseline and simulated
                policies.
            error_count (int):
                The number of log entries that could not be
                replayed.
            oldest_date (google.type.date_pb2.Date):
                The date of the oldest log entry replayed.
            newest_date (google.type.date_pb2.Date):
                The date of the newest log entry replayed.
        """

        log_count: int = proto.Field(
            proto.INT32,
            number=1,
        )
        unchanged_count: int = proto.Field(
            proto.INT32,
            number=2,
        )
        difference_count: int = proto.Field(
            proto.INT32,
            number=3,
        )
        error_count: int = proto.Field(
            proto.INT32,
            number=4,
        )
        oldest_date: date_pb2.Date = proto.Field(
            proto.MESSAGE,
            number=5,
            message=date_pb2.Date,
        )
        newest_date: date_pb2.Date = proto.Field(
            proto.MESSAGE,
            number=6,
            message=date_pb2.Date,
        )

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    state: State = proto.Field(
        proto.ENUM,
        number=2,
        enum=State,
    )
    config: "ReplayConfig" = proto.Field(
        proto.MESSAGE,
        number=3,
        message="ReplayConfig",
    )
    results_summary: ResultsSummary = proto.Field(
        proto.MESSAGE,
        number=5,
        message=ResultsSummary,
    )


class ReplayResult(proto.Message):
    r"""The result of replaying a single access tuple against a
    simulated state.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        diff (google.cloud.policysimulator_v1.types.ReplayDiff):
            The difference between the principal's access
            under the current (baseline) policies and the
            principal's access under the proposed
            (simulated) policies.

            This field is only included for access tuples
            that were successfully replayed and had
            different results under the current policies and
            the proposed policies.

            This field is a member of `oneof`_ ``result``.
        error (google.rpc.status_pb2.Status):
            The error that caused the access tuple replay
            to fail.
            This field is only included for access tuples
            that were not replayed successfully.

            This field is a member of `oneof`_ ``result``.
        name (str):
            The resource name of the ``ReplayResult``, in the following
            format:

            ``{projects|folders|organizations}/{resource-id}/locations/global/replays/{replay-id}/results/{replay-result-id}``,
            where ``{resource-id}`` is the ID of the project, folder, or
            organization that owns the
            [Replay][google.cloud.policysimulator.v1.Replay].

            Example:
            ``projects/my-example-project/locations/global/replays/506a5f7f-38ce-4d7d-8e03-479ce1833c36/results/1234``
        parent (str):
            The [Replay][google.cloud.policysimulator.v1.Replay] that
            the access tuple was included in.
        access_tuple (google.cloud.policysimulator_v1.types.AccessTuple):
            The access tuple that was replayed. This
            field includes information about the principal,
            resource, and permission that were involved in
            the access attempt.
        last_seen_date (google.type.date_pb2.Date):
            The latest date this access tuple was seen in
            the logs.
    """

    diff: "ReplayDiff" = proto.Field(
        proto.MESSAGE,
        number=5,
        oneof="result",
        message="ReplayDiff",
    )
    error: status_pb2.Status = proto.Field(
        proto.MESSAGE,
        number=6,
        oneof="result",
        message=status_pb2.Status,
    )
    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    parent: str = proto.Field(
        proto.STRING,
        number=2,
    )
    access_tuple: explanations.AccessTuple = proto.Field(
        proto.MESSAGE,
        number=3,
        message=explanations.AccessTuple,
    )
    last_seen_date: date_pb2.Date = proto.Field(
        proto.MESSAGE,
        number=4,
        message=date_pb2.Date,
    )


class CreateReplayRequest(proto.Message):
    r"""Request message for
    [Simulator.CreateReplay][google.cloud.policysimulator.v1.Simulator.CreateReplay].

    Attributes:
        parent (str):
            Required. The parent resource where this
            [Replay][google.cloud.policysimulator.v1.Replay] will be
            created. This resource must be a project, folder, or
            organization with a location.

            Example: ``projects/my-example-project/locations/global``
        replay (google.cloud.policysimulator_v1.types.Replay):
            Required. The
            [Replay][google.cloud.policysimulator.v1.Replay] to create.
            Set ``Replay.ReplayConfig`` to configure the replay.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    replay: "Replay" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="Replay",
    )


class ReplayOperationMetadata(proto.Message):
    r"""Metadata about a Replay operation.

    Attributes:
        start_time (google.protobuf.timestamp_pb2.Timestamp):
            Time when the request was received.
    """

    start_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=1,
        message=timestamp_pb2.Timestamp,
    )


class GetReplayRequest(proto.Message):
    r"""Request message for
    [Simulator.GetReplay][google.cloud.policysimulator.v1.Simulator.GetReplay].

    Attributes:
        name (str):
            Required. The name of the
            [Replay][google.cloud.policysimulator.v1.Replay] to
            retrieve, in the following format:

            ``{projects|folders|organizations}/{resource-id}/locations/global/replays/{replay-id}``,
            where ``{resource-id}`` is the ID of the project, folder, or
            organization that owns the ``Replay``.

            Example:
            ``projects/my-example-project/locations/global/replays/506a5f7f-38ce-4d7d-8e03-479ce1833c36``
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class ListReplayResultsRequest(proto.Message):
    r"""Request message for
    [Simulator.ListReplayResults][google.cloud.policysimulator.v1.Simulator.ListReplayResults].

    Attributes:
        parent (str):
            Required. The
            [Replay][google.cloud.policysimulator.v1.Replay] whose
            results are listed, in the following format:

            ``{projects|folders|organizations}/{resource-id}/locations/global/replays/{replay-id}``

            Example:
            ``projects/my-project/locations/global/replays/506a5f7f-38ce-4d7d-8e03-479ce1833c36``
        page_size (int):
            The maximum number of
            [ReplayResult][google.cloud.policysimulator.v1.ReplayResult]
            objects to return. Defaults to 5000.

            The maximum value is 5000; values above 5000 are rounded
            down to 5000.
        page_token (str):
            A page token, received from a previous
            [Simulator.ListReplayResults][google.cloud.policysimulator.v1.Simulator.ListReplayResults]
            call. Provide this token to retrieve the next page of
            results.

            When paginating, all other parameters provided to
            [Simulator.ListReplayResults[] must match the call that
            provided the page token.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=2,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=3,
    )


class ListReplayResultsResponse(proto.Message):
    r"""Response message for
    [Simulator.ListReplayResults][google.cloud.policysimulator.v1.Simulator.ListReplayResults].

    Attributes:
        replay_results (MutableSequence[google.cloud.policysimulator_v1.types.ReplayResult]):
            The results of running a
            [Replay][google.cloud.policysimulator.v1.Replay].
        next_page_token (str):
            A token that you can use to retrieve the next page of
            [ReplayResult][google.cloud.policysimulator.v1.ReplayResult]
            objects. If this field is omitted, there are no subsequent
            pages.
    """

    @property
    def raw_page(self):
        return self

    replay_results: MutableSequence["ReplayResult"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="ReplayResult",
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class ReplayConfig(proto.Message):
    r"""The configuration used for a
    [Replay][google.cloud.policysimulator.v1.Replay].

    Attributes:
        policy_overlay (MutableMapping[str, google.iam.v1.policy_pb2.Policy]):
            A mapping of the resources that you want to simulate
            policies for and the policies that you want to simulate.

            Keys are the full resource names for the resources. For
            example,
            ``//cloudresourcemanager.googleapis.com/projects/my-project``.
            For examples of full resource names for Google Cloud
            services, see
            https://cloud.google.com/iam/help/troubleshooter/full-resource-names.

            Values are [Policy][google.iam.v1.Policy] objects
            representing the policies that you want to simulate.

            Replays automatically take into account any IAM policies
            inherited through the resource hierarchy, and any policies
            set on descendant resources. You do not need to include
            these policies in the policy overlay.
        log_source (google.cloud.policysimulator_v1.types.ReplayConfig.LogSource):
            The logs to use as input for the
            [Replay][google.cloud.policysimulator.v1.Replay].
    """

    class LogSource(proto.Enum):
        r"""The source of the logs to use for a
        [Replay][google.cloud.policysimulator.v1.Replay].

        Values:
            LOG_SOURCE_UNSPECIFIED (0):
                An unspecified log source. If the log source is unspecified,
                the [Replay][google.cloud.policysimulator.v1.Replay]
                defaults to using ``RECENT_ACCESSES``.
            RECENT_ACCESSES (1):
                All access logs from the last 90 days. These
                logs may not include logs from the most recent 7
                days.
        """

        LOG_SOURCE_UNSPECIFIED = 0
        RECENT_ACCESSES = 1

    policy_overlay: MutableMapping[str, policy_pb2.Policy] = proto.MapField(
        proto.STRING,
        proto.MESSAGE,
        number=1,
        message=policy_pb2.Policy,
    )
    log_source: LogSource = proto.Field(
        proto.ENUM,
        number=2,
        enum=LogSource,
    )


class ReplayDiff(proto.Message):
    r"""The difference between the results of evaluating an access
    tuple under the current (baseline) policies and under the
    proposed (simulated) policies. This difference explains how a
    principal's access could change if the proposed policies were
    applied.

    Attributes:
        access_diff (google.cloud.policysimulator_v1.types.AccessStateDiff):
            A summary and comparison of the principal's access under the
            current (baseline) policies and the proposed (simulated)
            policies for a single access tuple.

            The evaluation of the principal's access is reported in the
            [AccessState][google.cloud.policysimulator.v1.AccessState]
            field.
    """

    access_diff: "AccessStateDiff" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="AccessStateDiff",
    )


class AccessStateDiff(proto.Message):
    r"""A summary and comparison of the principal's access under the
    current (baseline) policies and the proposed (simulated)
    policies for a single access tuple.

    Attributes:
        baseline (google.cloud.policysimulator_v1.types.ExplainedAccess):
            The results of evaluating the access tuple under the current
            (baseline) policies.

            If the
            [AccessState][google.cloud.policysimulator.v1.AccessState]
            couldn't be fully evaluated, this field explains why.
        simulated (google.cloud.policysimulator_v1.types.ExplainedAccess):
            The results of evaluating the access tuple
            under the proposed (simulated) policies.

            If the AccessState couldn't be fully evaluated,
            this field explains why.
        access_change (google.cloud.policysimulator_v1.types.AccessStateDiff.AccessChangeType):
            How the principal's access, specified in the
            AccessState field, changed between the current
            (baseline) policies and proposed (simulated)
            policies.
    """

    class AccessChangeType(proto.Enum):
        r"""How the principal's access, specified in the AccessState
        field, changed between the current (baseline) policies and
        proposed (simulated) policies.

        Values:
            ACCESS_CHANGE_TYPE_UNSPECIFIED (0):
                Default value. This value is unused.
            NO_CHANGE (1):
                The principal's access did not change.
                This includes the case where both baseline and
                simulated are UNKNOWN, but the unknown
                information is equivalent.
            UNKNOWN_CHANGE (2):
                The principal's access under both the current policies and
                the proposed policies is ``UNKNOWN``, but the unknown
                information differs between them.
            ACCESS_REVOKED (3):
                The principal had access under the current policies
                (``GRANTED``), but will no longer have access after the
                proposed changes (``NOT_GRANTED``).
            ACCESS_GAINED (4):
                The principal did not have access under the current policies
                (``NOT_GRANTED``), but will have access after the proposed
                changes (``GRANTED``).
            ACCESS_MAYBE_REVOKED (5):
                This result can occur for the following reasons:

                - The principal had access under the current policies
                  (``GRANTED``), but their access after the proposed changes
                  is ``UNKNOWN``.

                - The principal's access under the current policies is
                  ``UNKNOWN``, but they will not have access after the
                  proposed changes (``NOT_GRANTED``).
            ACCESS_MAYBE_GAINED (6):
                This result can occur for the following reasons:

                - The principal did not have access under the current
                  policies (``NOT_GRANTED``), but their access after the
                  proposed changes is ``UNKNOWN``.

                - The principal's access under the current policies is
                  ``UNKNOWN``, but they will have access after the proposed
                  changes (``GRANTED``).
        """

        ACCESS_CHANGE_TYPE_UNSPECIFIED = 0
        NO_CHANGE = 1
        UNKNOWN_CHANGE = 2
        ACCESS_REVOKED = 3
        ACCESS_GAINED = 4
        ACCESS_MAYBE_REVOKED = 5
        ACCESS_MAYBE_GAINED = 6

    baseline: "ExplainedAccess" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="ExplainedAccess",
    )
    simulated: "ExplainedAccess" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="ExplainedAccess",
    )
    access_change: AccessChangeType = proto.Field(
        proto.ENUM,
        number=3,
        enum=AccessChangeType,
    )


class ExplainedAccess(proto.Message):
    r"""Details about how a set of policies, listed in
    [ExplainedPolicy][google.cloud.policysimulator.v1.ExplainedPolicy],
    resulted in a certain
    [AccessState][google.cloud.policysimulator.v1.AccessState] when
    replaying an access tuple.

    Attributes:
        access_state (google.cloud.policysimulator_v1.types.AccessState):
            Whether the principal in the access tuple has
            permission to access the resource in the access
            tuple under the given policies.
        policies (MutableSequence[google.cloud.policysimulator_v1.types.ExplainedPolicy]):
            If the
            [AccessState][google.cloud.policysimulator.v1.AccessState]
            is ``UNKNOWN``, this field contains the policies that led to
            that result.

            If the ``AccessState`` is ``GRANTED`` or ``NOT_GRANTED``,
            this field is omitted.
        errors (MutableSequence[google.rpc.status_pb2.Status]):
            If the
            [AccessState][google.cloud.policysimulator.v1.AccessState]
            is ``UNKNOWN``, this field contains a list of errors
            explaining why the result is ``UNKNOWN``.

            If the ``AccessState`` is ``GRANTED`` or ``NOT_GRANTED``,
            this field is omitted.
    """

    access_state: explanations.AccessState = proto.Field(
        proto.ENUM,
        number=1,
        enum=explanations.AccessState,
    )
    policies: MutableSequence[explanations.ExplainedPolicy] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message=explanations.ExplainedPolicy,
    )
    errors: MutableSequence[status_pb2.Status] = proto.RepeatedField(
        proto.MESSAGE,
        number=3,
        message=status_pb2.Status,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

import google.iam.v1.policy_pb2 as policy_pb2  # type: ignore
import google.type.expr_pb2 as expr_pb2  # type: ignore
import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.cloud.policysimulator.v1",
    manifest={
        "AccessState",
        "HeuristicRelevance",
        "AccessTuple",
        "ExplainedPolicy",
        "BindingExplanation",
    },
)


class AccessState(proto.Enum):
    r"""Whether a principal has a permission for a resource.

    Values:
        ACCESS_STATE_UNSPECIFIED (0):
            Default value. This value is unused.
        GRANTED (1):
            The principal has the permission.
        NOT_GRANTED (2):
            The principal does not have the permission.
        UNKNOWN_CONDITIONAL (3):
            The principal has the permission only if a condition
            expression evaluates to ``true``.
        UNKNOWN_INFO_DENIED (4):
            The user who created the
            [Replay][google.cloud.policysimulator.v1.Replay] does not
            have access to all of the policies that Policy Simulator
            needs to evaluate.
    """

    ACCESS_STATE_UNSPECIFIED = 0
    GRANTED = 1
    NOT_GRANTED = 2
    UNKNOWN_CONDITIONAL = 3
    UNKNOWN_INFO_DENIED = 4


class HeuristicRelevance(proto.Enum):
    r"""The extent to which a single data point, such as the
    existence of a binding or whether a binding includes a specific
    principal, contributes to an overall determination.

    Values:
        HEURISTIC_RELEVANCE_UNSPECIFIED (0):
            Default value. This value is unused.
        NORMAL (1):
            The data point has a limited effect on the
            result. Changing the data point is unlikely to
            affect the overall determination.
        HIGH (2):
            The data point has a strong effect on the
            result. Changing the data point is likely to
            affect the overall determination.
    """

    HEURISTIC_RELEVANCE_UNSPECIFIED = 0
    NORMAL = 1
    HIGH = 2


class AccessTuple(proto.Message):
    r"""Information about the principal, resource, and permission to
    check.

    Attributes:
        principal (str):
            Required. The principal whose access you want to check, in
            the form of the email address that represents that
            principal. For example, ``alice@example.com`` or
            ``my-service-account@my-project.iam.gserviceaccount.com``.

            The principal must be a Google Account or a service account.
            Other types of principals are not supported.
        full_resource_name (str):
            Required. The full resource name that identifies the
            resource. For example,
            ``//compute.googleapis.com/projects/my-project/zones/us-central1-a/instances/my-instance``.

            For examples of full resource names for Google Cloud
            services, see
            https://cloud.google.com/iam/help/troubleshooter/full-resource-names.
        permission (str):
            Required. The IAM permission to check for the
            specified principal and resource.

            For a complete list of IAM permissions, see
            https://cloud.google.com/iam/help/permissions/reference.

            For a complete list of predefined IAM roles and
            the permissions in each role, see
            https://cloud.google.com/iam/help/roles/reference.
    """

    principal: str = proto.Field(
        proto.STRING,
        number=1,
    )
    full_resource_name: str = proto.Field(
        proto.STRING,
        number=2,
    )
    permission: str = proto.Field(
        proto.STRING,
        number=3,
    )


class ExplainedPolicy(proto.Message):
    r"""Details about how a specific IAM [Policy][google.iam.v1.Policy]
    contributed to the access check.

    Attributes:
        access (google.cloud.policysimulator_v1.types.AccessState):
            Indicates whether *this policy* provides the specified
            permission to the specified principal for the specified
            resource.

            This field does *not* indicate whether the principal
            actually has the permission for the resource. There might be
            another policy that overrides this policy. To determine
            whether the principal actually has the permission, use the
            ``access`` field in the
            [TroubleshootIamPolicyResponse][google.cloud.policytroubleshooter.v3.TroubleshootIamPolicyResponse].
        full_resource_name (str):
            The full resource name that identifies the resource. For
            example,
            ``//compute.googleapis.com/projects/my-project/zones/us-central1-a/instances/my-instance``.

            If the user who created the
            [Replay][google.cloud.policysimulator.v1.Replay] does not
            have access to the policy, this field is omitted.

            For examples of full resource names for Google Cloud
            services, see
            https://cloud.google.com/iam/help/troubleshooter/full-resource-names.
        policy (google.iam.v1.policy_pb2.Policy):
            The IAM policy attached to the resource.

            If the user who created the
            [Replay][google.cloud.policysimulator.v1.Replay] does not
            have access to the policy, this field is empty.
        binding_explanations (MutableSequence[google.cloud.policysimulator_v1.types.BindingExplanation]):
            Details about how each binding in the policy affects the
            principal's ability, or inability, to use the permission for
            the resource.

            If the user who created the
            [Replay][google.cloud.policysimulator.v1.Replay] does not
            have access to the policy, this field is omitted.
        relevance (google.cloud.policysimulator_v1.types.HeuristicRelevance):
            The relevance of this policy to the overall determination in
            the
            [TroubleshootIamPolicyResponse][google.cloud.policytroubleshooter.v3.TroubleshootIamPolicyResponse].

            If the user who created the
            [Replay][google.cloud.policysimulator.v1.Replay] does not
            have access to the policy, this field is omitted.
    """

    access: "AccessState" = proto.Field(
        proto.ENUM,
        number=1,
        enum="AccessState",
    )
    full_resource_name: str = proto.Field(
        proto.STRING,
        number=2,
    )
    policy: policy_pb2.Policy = proto.Field(
        proto.MESSAGE,
        number=3,
        message=policy_pb2.Policy,
    )
    binding_explanations: MutableSequence["BindingExplanation"] = proto.RepeatedField(
        proto.MESSAGE,
        number=4,
        message="BindingExplanation",
    )
    relevance: "HeuristicRelevance" = proto.Field(
        proto.ENUM,
        number=5,
        enum="HeuristicRelevance",
    )


class BindingExplanation(proto.Message):
    r"""Details about how a binding in a policy affects a principal's
    ability to use a permission.

    Attributes:
        access (google.cloud.policysimulator_v1.types.AccessState):
            Required. Indicates whether *this binding* provides the
            specified permission to the specified principal for the
            specified resource.

            This field does *not* indicate whether the principal
            actually has the permission for the resource. There might be
            another binding that overrides this binding. To determine
            whether the principal actually has the permission, use the
            ``access`` field in the
            [TroubleshootIamPolicyResponse][google.cloud.policytroubleshooter.v3.TroubleshootIamPolicyResponse].
        role (str):
            The role that this binding grants. For example,
            ``roles/compute.serviceAgent``.

            For a complete list of predefined IAM roles, as well as the
            permissions in each role, see
            https://cloud.google.com/iam/help/roles/reference.
        role_permission (google.cloud.policysimulator_v1.types.BindingExplanation.RolePermission):
            Indicates whether the role granted by this
            binding contains the specified permission.
        role_permission_relevance (google.cloud.policysimulator_v1.types.HeuristicRelevance):
            The relevance of the permission's existence,
            or nonexistence, in the role to the overall
            determination for the entire policy.
        memberships (MutableMapping[str, google.cloud.policysimulator_v1.types.BindingExplanation.AnnotatedMembership]):
            Indicates whether each principal in the binding includes the
            principal specified in the request, either directly or
            indirectly. Each key identifies a principal in the binding,
            and each value indicates whether the principal in the
            binding includes the principal in the request.

            For example, suppose that a binding includes the following
            principals:

            - ``user:alice@example.com``
            - ``group:product-eng@example.com``

            The principal in the replayed access tuple is
            ``user:bob@example.com``. This user is a principal of the
            group ``group:product-eng@example.com``.

            For the first principal in the binding, the key is
            ``user:alice@example.com``, and the ``membership`` field in
            the value is set to ``MEMBERSHIP_NOT_INCLUDED``.

            For the second principal in the binding, the key is
            ``group:product-eng@example.com``, and the ``membership``
            field in the value is set to ``MEMBERSHIP_INCLUDED``.
        relevance (google.cloud.policysimulator_v1.types.HeuristicRelevance):
            The relevance of this binding to the overall
            determination for the entire policy.
        condition (google.type.expr_pb2.Expr):
            A condition expression that prevents this binding from
            granting access unless the expression evaluates to ``true``.

            To learn about IAM Conditions, see
            https://cloud.google.com/iam/docs/conditions-overview.
    """

    class RolePermission(proto.Enum):
        r"""Whether a role includes a specific permission.

        Values:
            ROLE_PERMISSION_UNSPECIFIED (0):
                Default value. This value is unused.
            ROLE_PERMISSION_INCLUDED (1):
                The permission is included in the role.
            ROLE_PERMISSION_NOT_INCLUDED (2):
                The permission is not included in the role.
            ROLE_PERMISSION_UNKNOWN_INFO_DENIED (3):
                The user who created the
                [Replay][google.cloud.policysimulator.v1.Replay] is not
                allowed to access the binding.
        """

        ROLE_PERMISSION_UNSPECIFIED = 0
        ROLE_PERMISSION_INCLUDED = 1
        ROLE_PERMISSION_NOT_INCLUDED = 2
        ROLE_PERMISSION_UNKNOWN_INFO_DENIED = 3

    class Membership(proto.Enum):
        r"""Whether the binding includes the principal.

        Values:
            MEMBERSHIP_UNSPECIFIED (0):
                Default value. This value is unused.
            MEMBERSHIP_INCLUDED (1):
                The binding includes the principal. The principal can be
                included directly or indirectly. For example:

                - A principal is included directly if that principal is
                  listed in the binding.
                - A principal is included indirectly if that principal is in
                  a Google group or Google Workspace domain that is listed
                  in the binding.
            MEMBERSHIP_NOT_INCLUDED (2):
                The binding does not include the principal.
            MEMBERSHIP_UNKNOWN_INFO_DENIED (3):
                The user who created the
                [Replay][google.cloud.policysimulator.v1.Replay] is not
                allowed to access the binding.
            MEMBERSHIP_UNKNOWN_UNSUPPORTED (4):
                The principal is an unsupported type. Only
                Google Accounts and service accounts are
                supported.
        """

        MEMBERSHIP_UNSPECIFIED = 0
        MEMBERSHIP_INCLUDED = 1
        MEMBERSHIP_NOT_INCLUDED = 2
        MEMBERSHIP_UNKNOWN_INFO_DENIED = 3
        MEMBERSHIP_UNKNOWN_UNSUPPORTED = 4

    class AnnotatedMembership(proto.Message):
        r"""Details about whether the binding includes the principal.

        Attributes:
            membership (google.cloud.policysimulator_v1.types.BindingExplanation.Membership):
                Indicates whether the binding includes the
                principal.
            relevance (google.cloud.policysimulator_v1.types.HeuristicRelevance):
                The relevance of the principal's status to
                the overall determination for the binding.
        """

        membership: "BindingExplanation.Membership" = proto.Field(
            proto.ENUM,
            number=1,
            enum="BindingExplanation.Membership",
        )
        relevance: "HeuristicRelevance" = proto.Field(
            proto.ENUM,
            number=2,
            enum="HeuristicRelevance",
        )

    access: "AccessState" = proto.Field(
        proto.ENUM,
        number=1,
        enum="AccessState",
    )
    role: str = proto.Field(
        proto.STRING,
        number=2,
    )
    role_permission: RolePermission = proto.Field(
        proto.ENUM,
        number=3,
        enum=RolePermission,
    )
    role_permission_relevance: "HeuristicRelevance" = proto.Field(
        proto.ENUM,
        number=4,
        enum="HeuristicRelevance",
    )
    memberships: MutableMapping[str, AnnotatedMembership] = proto.MapField(
        proto.STRING,
        proto.MESSAGE,
        number=5,
        message=AnnotatedMembership,
    )
    relevance: "HeuristicRelevance" = proto.Field(
        proto.ENUM,
        number=6,
        enum="HeuristicRelevance",
    )
    condition: expr_pb2.Expr = proto.Field(
        proto.MESSAGE,
        number=7,
        message=expr_pb2.Expr,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import logging as std_logging
import re
import uuid
from collections import OrderedDict
from typing import (
    Callable,
    Dict,
    Mapping,
    MutableMapping,
    MutableSequence,
    Optional,
    Sequence,
    Tuple,
    Type,
    Union,
)

import google.protobuf
from google.api_core import exceptions as core_exceptions
from google.api_core import gapic_v1
from google.api_core import retry_async as retries
from google.api_core.client_options import ClientOptions
from google.auth import credentials as ga_credentials  # type: ignore
from google.oauth2 import service_account  # type: ignore

from google.cloud.policysimulator_v1 import gapic_version as package_version

try:
    OptionalRetry = Union[retries.AsyncRetry, gapic_v1.method._MethodDefault, None]
except AttributeError:  # pragma: NO COVER
    OptionalRetry = Union[retries.AsyncRetry, object, None]  # type: ignore

import google.api_core.operation as operation  # type: ignore
import google.api_core.operation_async as operation_async  # type: ignore
import google.protobuf.timestamp_pb2 as timestamp_pb2  # type: ignore
from google.longrunning import operations_pb2  # type: ignore

from google.cloud.policysimulator_v1.services.org_policy_violations_preview_service import (
    pagers,
)
from google.cloud.policysimulator_v1.types import orgpolicy as gcp_orgpolicy

from .client import OrgPolicyViolationsPreviewServiceClient
from .transports.base import (
    DEFAULT_CLIENT_INFO,
    OrgPolicyViolationsPreviewServiceTransport,
)
from .transports.grpc_asyncio import (
    OrgPolicyViolationsPreviewServiceGrpcAsyncIOTransport,
)

try:
    from google.api_core import client_logging  # type: ignore

    CLIENT_LOGGING_SUPPORTED = True  # pragma: NO COVER
except ImportError:  # pragma: NO COVER
    CLIENT_LOGGING_SUPPORTED = False

_LOGGER = std_logging.getLogger(__name__)


class OrgPolicyViolationsPreviewServiceAsyncClient:
    """Violations Preview API service for OrgPolicy.

    An
    [OrgPolicyViolationsPreview][google.cloud.policysimulator.v1.OrgPolicyViolationsPreview]
    is a preview of the violations that will exist as soon as a proposed
    OrgPolicy change is submitted. To create an
    [OrgPolicyViolationsPreview][google.cloud.policysimulator.v1.OrgPolicyViolationsPreview],
    the API user specifies the changes they wish to make and requests
    the generation of a preview via [GenerateViolationsPreview][]. the
    OrgPolicy Simulator service then scans the API user's currently
    existing resources to determine these resources violate the newly
    set OrgPolicy.
    """

    _client: OrgPolicyViolationsPreviewServiceClient

    # Copy defaults from the synchronous client for use here.
    # Note: DEFAULT_ENDPOINT is deprecated. Use _DEFAULT_ENDPOINT_TEMPLATE instead.
    DEFAULT_ENDPOINT = OrgPolicyViolationsPreviewServiceClient.DEFAULT_ENDPOINT
    DEFAULT_MTLS_ENDPOINT = (
        OrgPolicyViolationsPreviewServiceClient.DEFAULT_MTLS_ENDPOINT
    )
    _DEFAULT_ENDPOINT_TEMPLATE = (
        OrgPolicyViolationsPreviewServiceClient._DEFAULT_ENDPOINT_TEMPLATE
    )
    _DEFAULT_UNIVERSE = OrgPolicyViolationsPreviewServiceClient._DEFAULT_UNIVERSE

    custom_constraint_path = staticmethod(
        OrgPolicyViolationsPreviewServiceClient.custom_constraint_path
    )
    parse_custom_constraint_path = staticmethod(
        OrgPolicyViolationsPreviewServiceClient.parse_custom_constraint_path
    )
    org_policy_violation_path = staticmethod(
        OrgPolicyViolationsPreviewServiceClient.org_policy_violation_path
    )
    parse_org_policy_violation_path = staticmethod(
        OrgPolicyViolationsPreviewServiceClient.parse_org_policy_violation_path
    )
    org_policy_violations_preview_path = staticmethod(
        OrgPolicyViolationsPreviewServiceClient.org_policy_violations_preview_path
    )
    parse_org_policy_violations_preview_path = staticmethod(
        OrgPolicyViolationsPreviewServiceClient.parse_org_policy_violations_preview_path
    )
    policy_path = staticmethod(OrgPolicyViolationsPreviewServiceClient.policy_path)
    parse_policy_path = staticmethod(
        OrgPolicyViolationsPreviewServiceClient.parse_policy_path
    )
    common_billing_account_path = staticmethod(
        OrgPolicyViolationsPreviewServiceClient.common_billing_account_path
    )
    parse_common_billing_account_path = staticmethod(
        OrgPolicyViolationsPreviewServiceClient.parse_common_billing_account_path
    )
    common_folder_path = staticmethod(
        OrgPolicyViolationsPreviewServiceClient.common_folder_path
    )
    parse_common_folder_path = staticmethod(
        OrgPolicyViolationsPreviewServiceClient.parse_common_folder_path
    )
    common_organization_path = staticmethod(
        OrgPolicyViolationsPreviewServiceClient.common_organization_path
    )
    parse_common_organization_path = staticmethod(
        OrgPolicyViolationsPreviewServiceClient.parse_common_organization_path
    )
    common_project_path = staticmethod(
        OrgPolicyViolationsPreviewServiceClient.common_project_path
    )
    parse_common_project_path = staticmethod(
        OrgPolicyViolationsPreviewServiceClient.parse_common_project_path
    )
    common_location_path = staticmethod(
        OrgPolicyViolationsPreviewServiceClient.common_location_path
    )
    parse_common_location_path = staticmethod(
        OrgPolicyViolationsPreviewServiceClient.parse_common_location_path
    )

    @classmethod
    def from_service_account_info(cls, info: dict, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            info.

        Args:
            info (dict): The service account private key info.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            OrgPolicyViolationsPreviewServiceAsyncClient: The constructed client.
        """
        sa_info_func = (
            OrgPolicyViolationsPreviewServiceClient.from_service_account_info.__func__  # type: ignore
        )
        return sa_info_func(
            OrgPolicyViolationsPreviewServiceAsyncClient, info, *args, **kwargs
        )

    @classmethod
    def from_service_account_file(cls, filename: str, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            file.

        Args:
            filename (str): The path to the service account private key json
                file.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            OrgPolicyViolationsPreviewServiceAsyncClient: The constructed client.
        """
        sa_file_func = (
            OrgPolicyViolationsPreviewServiceClient.from_service_account_file.__func__  # type: ignore
        )
        return sa_file_func(
            OrgPolicyViolationsPreviewServiceAsyncClient, filename, *args, **kwargs
        )

    from_service_account_json = from_service_account_file

    @classmethod
    def get_mtls_endpoint_and_cert_source(
        cls, client_options: Optional[ClientOptions] = None
    ):
        """Return the API endpoint and client cert source for mutual TLS.

        The client cert source is determined in the following order:
        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
        client cert source is None.
        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
        default client cert source exists, use the default one; otherwise the client cert
        source is None.

        The API endpoint is determined in the following order:
        (1) if `client_options.api_endpoint` if provided, use the provided one.
        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
        default mTLS endpoint; if the environment variable is "never", use the default API
        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
        use the default API endpoint.

        More details can be found at https://google.aip.dev/auth/4114.

        Args:
            client_options (google.api_core.client_options.ClientOptions): Custom options for the
                client. Only the `api_endpoint` and `client_cert_source` properties may be used
                in this method.

        Returns:
            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
                client cert source to use.

        Raises:
            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
        """
        return (
            OrgPolicyViolationsPreviewServiceClient.get_mtls_endpoint_and_cert_source(
                client_options
            )
        )  # type: ignore

    @property
    def transport(self) -> OrgPolicyViolationsPreviewServiceTransport:
        """Returns the transport used by the client instance.

        Returns:
            OrgPolicyViolationsPreviewServiceTransport: The transport used by the client instance.
        """
        return self._client.transport

    @property
    def api_endpoint(self) -> str:
        """Return the API endpoint used by the client instance.

        Returns:
            str: The API endpoint used by the client instance.
        """
        return self._client._api_endpoint

    @property
    def universe_domain(self) -> str:
        """Return the universe domain used by the client instance.

        Returns:
            str: The universe domain used
                by the client instance.
        """
        return self._client._universe_domain

    get_transport_class = OrgPolicyViolationsPreviewServiceClient.get_transport_class

    def __init__(
        self,
        *,
        credentials: Optional[ga_credentials.Credentials] = None,
        transport: Optional[
            Union[
                str,
                OrgPolicyViolationsPreviewServiceTransport,
                Callable[..., OrgPolicyViolationsPreviewServiceTransport],
            ]
        ] = "grpc_asyncio",
        client_options: Optional[ClientOptions] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
    ) -> None:
        """Instantiates the org policy violations preview service async client.

        Args:
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            transport (Optional[Union[str,OrgPolicyViolationsPreviewServiceTransport,Callable[..., OrgPolicyViolationsPreviewServiceTransport]]]):
                The transport to use, or a Callable that constructs and returns a new transport to use.
                If a Callable is given, it will be called with the same set of initialization
                arguments as used in the OrgPolicyViolationsPreviewServiceTransport constructor.
                If set to None, a transport is chosen automatically.
            client_options (Optional[Union[google.api_core.client_options.ClientOptions, dict]]):
                Custom options for the client.

                1. The ``api_endpoint`` property can be used to override the
                default endpoint provided by the client when ``transport`` is
                not explicitly provided. Only if this property is not set and
                ``transport`` was not explicitly provided, the endpoint is
                determined by the GOOGLE_API_USE_MTLS_ENDPOINT environment
                variable, which have one of the following values:
                "always" (always use the default mTLS endpoint), "never" (always
                use the default regular endpoint) and "auto" (auto-switch to the
                default mTLS endpoint if client certificate is present; this is
                the default value).

                2. If the GOOGLE_API_USE_CLIENT_CERTIFICATE environment variable
                is "true", then the ``client_cert_source`` property can be used
                to provide a client certificate for mTLS transport. If
                not provided, the default SSL client certificate will be used if
                present. If GOOGLE_API_USE_CLIENT_CERTIFICATE is "false" or not
                set, no client certificate will be used.

                3. The ``universe_domain`` property can be used to override the
                default "googleapis.com" universe. Note that ``api_endpoint``
                property still takes precedence; and ``universe_domain`` is
                currently not supported for mTLS.

            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you're developing
                your own client library.

        Raises:
            google.auth.exceptions.MutualTlsChannelError: If mutual TLS transport
                creation failed for any reason.
        """
        self._client = OrgPolicyViolationsPreviewServiceClient(
            credentials=credentials,
            transport=transport,
            client_options=client_options,
            client_info=client_info,
        )

        if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
            std_logging.DEBUG
        ):  # pragma: NO COVER
            _LOGGER.debug(
                "Created client `google.cloud.policysimulator_v1.OrgPolicyViolationsPreviewServiceAsyncClient`.",
                extra={
                    "serviceName": "google.cloud.policysimulator.v1.OrgPolicyViolationsPreviewService",
                    "universeDomain": getattr(
                        self._client._transport._credentials, "universe_domain", ""
                    ),
                    "credentialsType": f"{type(self._client._transport._credentials).__module__}.{type(self._client._transport._credentials).__qualname__}",
                    "credentialsInfo": getattr(
                        self.transport._credentials, "get_cred_info", lambda: None
                    )(),
                }
                if hasattr(self._client._transport, "_credentials")
                else {
                    "serviceName": "google.cloud.policysimulator.v1.OrgPolicyViolationsPreviewService",
                    "credentialsType": None,
                },
            )

    async def list_org_policy_violations_previews(
        self,
        request: Optional[
            Union[gcp_orgpolicy.ListOrgPolicyViolationsPreviewsRequest, dict]
        ] = None,
        *,
        parent: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> pagers.ListOrgPolicyViolationsPreviewsAsyncPager:
        r"""ListOrgPolicyViolationsPreviews lists each
        [OrgPolicyViolationsPreview][google.cloud.policysimulator.v1.OrgPolicyViolationsPreview]
        in an organization. Each
        [OrgPolicyViolationsPreview][google.cloud.policysimulator.v1.OrgPolicyViolationsPreview]
        is available for at least 7 days.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import policysimulator_v1

            async def sample_list_org_policy_violations_previews():
                # Create a client
                client = policysimulator_v1.OrgPolicyViolationsPreviewServiceAsyncClient()

                # Initialize request argument(s)
                request = policysimulator_v1.ListOrgPolicyViolationsPreviewsRequest(
                    parent="parent_value",
                )

                # Make the request
                page_result = client.list_org_policy_violations_previews(request=request)

                # Handle the response
                async for response in page_result:
                    print(response)

        Args:
            request (Optional[Union[google.cloud.policysimulator_v1.types.ListOrgPolicyViolationsPreviewsRequest, dict]]):
                The request object. ListOrgPolicyViolationsPreviewsRequest is the request
                message for
                [OrgPolicyViolationsPreviewService.ListOrgPolicyViolationsPreviews][google.cloud.policysimulator.v1.OrgPolicyViolationsPreviewService.ListOrgPolicyViolationsPreviews].
            parent (:class:`str`):
                Required. The parent the violations are scoped to.
                Format:
                ``organizations/{organization}/locations/{location}``

                Example:
                ``organizations/my-example-org/locations/global``

                This corresponds to the ``parent`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.cloud.policysimulator_v1.services.org_policy_violations_preview_service.pagers.ListOrgPolicyViolationsPreviewsAsyncPager:
                ListOrgPolicyViolationsPreviewsResponse is the response message for
                   [OrgPolicyViolationsPreviewService.ListOrgPolicyViolationsPreviews][google.cloud.policysimulator.v1.OrgPolicyViolationsPreviewService.ListOrgPolicyViolationsPreviews].

                Iterating over this object will yield results and
                resolve additional pages automatically.

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [parent]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(
            request, gcp_orgpolicy.ListOrgPolicyViolationsPreviewsRequest
        ):
            request = gcp_orgpolicy.ListOrgPolicyViolationsPreviewsRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if parent is not None:
            request.parent = parent

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.list_org_policy_violations_previews
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # This method is paged; wrap the response in a pager, which provides
        # an `__aiter__` convenience method.
        response = pagers.ListOrgPolicyViolationsPreviewsAsyncPager(
            method=rpc,
            request=request,
            response=response,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def get_org_policy_violations_preview(
        self,
        request: Optional[
            Union[gcp_orgpolicy.GetOrgPolicyViolationsPreviewRequest, dict]
        ] = None,
        *,
        name: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> gcp_orgpolicy.OrgPolicyViolationsPreview:
        r"""GetOrgPolicyViolationsPreview gets the specified
        [OrgPolicyViolationsPreview][google.cloud.policysimulator.v1.OrgPolicyViolationsPreview].
        Each
        [OrgPolicyViolationsPreview][google.cloud.policysimulator.v1.OrgPolicyViolationsPreview]
        is available for at least 7 days.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import policysimulator_v1

            async def sample_get_org_policy_violations_preview():
                # Create a client
                client = policysimulator_v1.OrgPolicyViolationsPreviewServiceAsyncClient()

                # Initialize request argument(s)
                request = policysimulator_v1.GetOrgPolicyViolationsPreviewRequest(
                    name="name_value",
                )

                # Make the request
                response = await client.get_org_policy_violations_preview(request=request)

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.cloud.policysimulator_v1.types.GetOrgPolicyViolationsPreviewRequest, dict]]):
                The request object. GetOrgPolicyViolationsPreviewRequest is the request
                message for
                [OrgPolicyViolationsPreviewService.GetOrgPolicyViolationsPreview][google.cloud.policysimulator.v1.OrgPolicyViolationsPreviewService.GetOrgPolicyViolationsPreview].
            name (:class:`str`):
                Required. The name of the
                OrgPolicyViolationsPreview to get.

                This corresponds to the ``name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.cloud.policysimulator_v1.types.OrgPolicyViolationsPreview:
                OrgPolicyViolationsPreview is a resource providing a preview of the
                   violations that will exist if an OrgPolicy change is
                   made.

                   The list of violations are modeled as child resources
                   and retrieved via a [ListOrgPolicyViolations][] API
                   call. There are potentially more
                   [OrgPolicyViolations][] than could fit in an embedded
                   field. Thus, the use of a child resource instead of a
                   field.

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [name]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, gcp_orgpolicy.GetOrgPolicyViolationsPreviewRequest):
            request = gcp_orgpolicy.GetOrgPolicyViolationsPreviewRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if name is not None:
            request.name = name

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.get_org_policy_violations_preview
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def create_org_policy_violations_preview(
        self,
        request: Optional[
            Union[gcp_orgpolicy.CreateOrgPolicyViolationsPreviewRequest, dict]
        ] = None,
        *,
        parent: Optional[str] = None,
        org_policy_violations_preview: Optional[
            gcp_orgpolicy.OrgPolicyViolationsPreview
        ] = None,
        org_policy_violations_preview_id: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> operation_async.AsyncOperation:
        r"""CreateOrgPolicyViolationsPreview creates an
        [OrgPolicyViolationsPreview][google.cloud.policysimulator.v1.OrgPolicyViolationsPreview]
        for the proposed changes in the provided
        [OrgPolicyViolationsPreview.OrgPolicyOverlay][]. The changes to
        OrgPolicy are specified by this ``OrgPolicyOverlay``. The
        resources to scan are inferred from these specified changes.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import policysimulator_v1

            async def sample_create_org_policy_violations_preview():
                # Create a client
                client = policysimulator_v1.OrgPolicyViolationsPreviewServiceAsyncClient()

                # Initialize request argument(s)
                request = policysimulator_v1.CreateOrgPolicyViolationsPreviewRequest(
                    parent="parent_value",
                )

                # Make the request
                operation = client.create_org_policy_violations_preview(request=request)

                print("Waiting for operation to complete...")

                response = (await operation).result()

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.cloud.policysimulator_v1.types.CreateOrgPolicyViolationsPreviewRequest, dict]]):
                The request object. CreateOrgPolicyViolationsPreviewRequest is the request
                message for
                [OrgPolicyViolationsPreviewService.CreateOrgPolicyViolationsPreview][google.cloud.policysimulator.v1.OrgPolicyViolationsPreviewService.CreateOrgPolicyViolationsPreview].
            parent (:class:`str`):
                Required. The organization under which this
                [OrgPolicyViolationsPreview][google.cloud.policysimulator.v1.OrgPolicyViolationsPreview]
                will be created.

                Example:
                ``organizations/my-example-org/locations/global``

                This corresponds to the ``parent`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            org_policy_violations_preview (:class:`google.cloud.policysimulator_v1.types.OrgPolicyViolationsPreview`):
                Required. The
                [OrgPolicyViolationsPreview][google.cloud.policysimulator.v1.OrgPolicyViolationsPreview]
                to generate.

                This corresponds to the ``org_policy_violations_preview`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            org_policy_violations_preview_id (:class:`str`):
                Optional. An optional user-specified ID for the
                [OrgPolicyViolationsPreview][google.cloud.policysimulator.v1.OrgPolicyViolationsPreview].
                If not provided, a random ID will be generated.

                This corresponds to the ``org_policy_violations_preview_id`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.api_core.operation_async.AsyncOperation:
                An object representing a long-running operation.

                The result type for the operation will be :class:`google.cloud.policysimulator_v1.types.OrgPolicyViolationsPreview` OrgPolicyViolationsPreview is a resource providing a preview of the
                   violations that will exist if an OrgPolicy change is
                   made.

                   The list of violations are modeled as child resources
                   and retrieved via a [ListOrgPolicyViolations][] API
                   call. There are potentially more
                   [OrgPolicyViolations][] than could fit in an embedded
                   field. Thus, the use of a child resource instead of a
                   field.

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [
            parent,
            org_policy_violations_preview,
            org_policy_violations_preview_id,
        ]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(
            request, gcp_orgpolicy.CreateOrgPolicyViolationsPreviewRequest
        ):
            request = gcp_orgpolicy.CreateOrgPolicyViolationsPreviewRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if parent is not None:
            request.parent = parent
        if org_policy_violations_preview is not None:
            request.org_policy_violations_preview = org_policy_violations_preview
        if org_policy_violations_preview_id is not None:
            request.org_policy_violations_preview_id = org_policy_violations_preview_id

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.create_org_policy_violations_preview
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Wrap the response in an operation future.
        response = operation_async.from_gapic(
            response,
            self._client._transport.operations_client,
            gcp_orgpolicy.OrgPolicyViolationsPreview,
            metadata_type=gcp_orgpolicy.CreateOrgPolicyViolationsPreviewOperationMetadata,
        )

        # Done; return the response.
        return response

    async def list_org_policy_violations(
        self,
        request: Optional[
            Union[gcp_orgpolicy.ListOrgPolicyViolationsRequest, dict]
        ] = None,
        *,
        parent: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> pagers.ListOrgPolicyViolationsAsyncPager:
        r"""ListOrgPolicyViolations lists the [OrgPolicyViolations][] that
        are present in an
        [OrgPolicyViolationsPreview][google.cloud.policysimulator.v1.OrgPolicyViolationsPreview].

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import policysimulator_v1

            async def sample_list_org_policy_violations():
                # Create a client
                client = policysimulator_v1.OrgPolicyViolationsPreviewServiceAsyncClient()

                # Initialize request argument(s)
                request = policysimulator_v1.ListOrgPolicyViolationsRequest(
                    parent="parent_value",
                )

                # Make the request
                page_result = client.list_org_policy_violations(request=request)

                # Handle the response
                async for response in page_result:
                    print(response)

        Args:
            request (Optional[Union[google.cloud.policysimulator_v1.types.ListOrgPolicyViolationsRequest, dict]]):
                The request object. ListOrgPolicyViolationsRequest is the request message
                for
                [OrgPolicyViolationsPreviewService.ListOrgPolicyViolations][google.cloud.policysimulator.v1.OrgPolicyViolationsPreviewService.ListOrgPolicyViolations].
            parent (:class:`str`):
                Required. The
                OrgPolicyViolationsPreview to get
                OrgPolicyViolations from. Format:

                organizations/{organization}/locations/{location}/orgPolicyViolationsPreviews/{orgPolicyViolationsPreview}

                This corresponds to the ``parent`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.cloud.policysimulator_v1.services.org_policy_violations_preview_service.pagers.ListOrgPolicyViolationsAsyncPager:
                ListOrgPolicyViolationsResponse is the response message for
                   [OrgPolicyViolationsPreviewService.ListOrgPolicyViolations][google.cloud.policysimulator.v1.OrgPolicyViolationsPreviewService.ListOrgPolicyViolations]

                Iterating over this object will yield results and
                resolve additional pages automatically.

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [parent]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, gcp_orgpolicy.ListOrgPolicyViolationsRequest):
            request = gcp_orgpolicy.ListOrgPolicyViolationsRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if parent is not None:
            request.parent = parent

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.list_org_policy_violations
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # This method is paged; wrap the response in a pager, which provides
        # an `__aiter__` convenience method.
        response = pagers.ListOrgPolicyViolationsAsyncPager(
            method=rpc,
            request=request,
            response=response,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def list_operations(
        self,
        request: Optional[Union[operations_pb2.ListOperationsRequest, dict]] = None,
        *,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> operations_pb2.ListOperationsResponse:
        r"""Lists operations that match the specified filter in the request.

        Args:
            request (:class:`~.operations_pb2.ListOperationsRequest`):
                The request object. Request message for
                `ListOperations` method.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors,
                    if any, should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.
        Returns:
            ~.operations_pb2.ListOperationsResponse:
                Response message for ``ListOperations`` method.
        """
        # Create or coerce a protobuf request object.
        # The request isn't a proto-plus wrapped type,
        # so it must be constructed via keyword expansion.
        if request is None:
            request_pb = operations_pb2.ListOperationsRequest()
        elif isinstance(request, dict):
            request_pb = operations_pb2.ListOperationsRequest(**request)
        else:
            request_pb = request

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self.transport._wrapped_methods[self._client._transport.list_operations]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request_pb.name),)),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request_pb,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def get_operation(
        self,
        request: Optional[Union[operations_pb2.GetOperationRequest, dict]] = None,
        *,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> operations_pb2.Operation:
        r"""Gets the latest state of a long-running operation.

        Args:
            request (:class:`~.operations_pb2.GetOperationRequest`):
                The request object. Request message for
                `GetOperation` method.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors,
                    if any, should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.
        Returns:
            ~.operations_pb2.Operation:
                An ``Operation`` object.
        """
        # Create or coerce a protobuf request object.
        # The request isn't a proto-plus wrapped type,
        # so it must be constructed via keyword expansion.
        if request is None:
            request_pb = operations_pb2.GetOperationRequest()
        elif isinstance(request, dict):
            request_pb = operations_pb2.GetOperationRequest(**request)
        else:
            request_pb = request

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self.transport._wrapped_methods[self._client._transport.get_operation]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request_pb.name),)),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request_pb,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def __aenter__(self) -> "OrgPolicyViolationsPreviewServiceAsyncClient":
        return self

    async def __aexit__(self, exc_type, exc, tb):
        await self.transport.close()


DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
    gapic_version=package_version.__version__
)

if hasattr(DEFAULT_CLIENT_INFO, "protobuf_runtime_version"):  # pragma: NO COVER
    DEFAULT_CLIENT_INFO.protobuf_runtime_version = google.protobuf.__version__


__all__ = ("OrgPolicyViolationsPreviewServiceAsyncClient",)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from google.cloud.policysimulator import gapic_version as package_version

__version__ = package_version.__version__


from google.cloud.policysimulator_v1.services.org_policy_violations_preview_service.async_client import (
    OrgPolicyViolationsPreviewServiceAsyncClient,
)
from google.cloud.policysimulator_v1.services.org_policy_violations_preview_service.client import (
    OrgPolicyViolationsPreviewServiceClient,
)
from google.cloud.policysimulator_v1.services.simulator.async_client import (
    SimulatorAsyncClient,
)
from google.cloud.policysimulator_v1.services.simulator.client import SimulatorClient
from google.cloud.policysimulator_v1.types.explanations import (
    AccessState,
    AccessTuple,
    BindingExplanation,
    ExplainedPolicy,
    HeuristicRelevance,
)
from google.cloud.policysimulator_v1.types.orgpolicy import (
    CreateOrgPolicyViolationsPreviewOperationMetadata,
    CreateOrgPolicyViolationsPreviewRequest,
    GetOrgPolicyViolationsPreviewRequest,
    ListOrgPolicyViolationsPreviewsRequest,
    ListOrgPolicyViolationsPreviewsResponse,
    ListOrgPolicyViolationsRequest,
    ListOrgPolicyViolationsResponse,
    OrgPolicyOverlay,
    OrgPolicyViolation,
    OrgPolicyViolationsPreview,
    PreviewState,
    ResourceContext,
)
from google.cloud.policysimulator_v1.types.simulator import (
    AccessStateDiff,
    CreateReplayRequest,
    ExplainedAccess,
    GetReplayRequest,
    ListReplayResultsRequest,
    ListReplayResultsResponse,
    Replay,
    ReplayConfig,
    ReplayDiff,
    ReplayOperationMetadata,
    ReplayResult,
)

__all__ = (
    "OrgPolicyViolationsPreviewServiceClient",
    "OrgPolicyViolationsPreviewServiceAsyncClient",
    "SimulatorClient",
    "SimulatorAsyncClient",
    "AccessTuple",
    "BindingExplanation",
    "ExplainedPolicy",
    "AccessState",
    "HeuristicRelevance",
    "CreateOrgPolicyViolationsPreviewOperationMetadata",
    "CreateOrgPolicyViolationsPreviewRequest",
    "GetOrgPolicyViolationsPreviewRequest",
    "ListOrgPolicyViolationsPreviewsRequest",
    "ListOrgPolicyViolationsPreviewsResponse",
    "ListOrgPolicyViolationsRequest",
    "ListOrgPolicyViolationsResponse",
    "OrgPolicyOverlay",
    "OrgPolicyViolation",
    "OrgPolicyViolationsPreview",
    "ResourceContext",
    "PreviewState",
    "AccessStateDiff",
    "CreateReplayRequest",
    "ExplainedAccess",
    "GetReplayRequest",
    "ListReplayResultsRequest",
    "ListReplayResultsResponse",
    "Replay",
    "ReplayConfig",
    "ReplayDiff",
    "ReplayOperationMetadata",
    "ReplayResult",
)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

import google.protobuf.timestamp_pb2 as timestamp_pb2  # type: ignore
import google.rpc.status_pb2 as status_pb2  # type: ignore
import proto  # type: ignore
from google.cloud.orgpolicy_v2.types import constraint
from google.cloud.orgpolicy_v2.types import orgpolicy as gcov_orgpolicy

__protobuf__ = proto.module(
    package="google.cloud.policysimulator.v1",
    manifest={
        "PreviewState",
        "OrgPolicyViolationsPreview",
        "OrgPolicyViolation",
        "ResourceContext",
        "OrgPolicyOverlay",
        "CreateOrgPolicyViolationsPreviewOperationMetadata",
        "ListOrgPolicyViolationsPreviewsRequest",
        "ListOrgPolicyViolationsPreviewsResponse",
        "GetOrgPolicyViolationsPreviewRequest",
        "CreateOrgPolicyViolationsPreviewRequest",
        "ListOrgPolicyViolationsRequest",
        "ListOrgPolicyViolationsResponse",
    },
)


class PreviewState(proto.Enum):
    r"""The current state of an
    [OrgPolicyViolationsPreview][google.cloud.policysimulator.v1.OrgPolicyViolationsPreview].

    Values:
        PREVIEW_STATE_UNSPECIFIED (0):
            The state is unspecified.
        PREVIEW_PENDING (1):
            The
            [OrgPolicyViolationsPreview][google.cloud.policysimulator.v1.OrgPolicyViolationsPreview]
            has not been created yet.
        PREVIEW_RUNNING (2):
            The
            [OrgPolicyViolationsPreview][google.cloud.policysimulator.v1.OrgPolicyViolationsPreview]
            is currently being created.
        PREVIEW_SUCCEEDED (3):
            The
            [OrgPolicyViolationsPreview][google.cloud.policysimulator.v1.OrgPolicyViolationsPreview]
            creation finished successfully.
        PREVIEW_FAILED (4):
            The
            [OrgPolicyViolationsPreview][google.cloud.policysimulator.v1.OrgPolicyViolationsPreview]
            creation failed with an error.
    """

    PREVIEW_STATE_UNSPECIFIED = 0
    PREVIEW_PENDING = 1
    PREVIEW_RUNNING = 2
    PREVIEW_SUCCEEDED = 3
    PREVIEW_FAILED = 4


class OrgPolicyViolationsPreview(proto.Message):
    r"""OrgPolicyViolationsPreview is a resource providing a preview of the
    violations that will exist if an OrgPolicy change is made.

    The list of violations are modeled as child resources and retrieved
    via a [ListOrgPolicyViolations][] API call. There are potentially
    more [OrgPolicyViolations][] than could fit in an embedded field.
    Thus, the use of a child resource instead of a field.

    Attributes:
        name (str):
            Output only. The resource name of the
            ``OrgPolicyViolationsPreview``. It has the following format:

            ``organizations/{organization}/locations/{location}/orgPolicyViolationsPreviews/{orgPolicyViolationsPreview}``

            Example:
            ``organizations/my-example-org/locations/global/orgPolicyViolationsPreviews/506a5f7f``
        state (google.cloud.policysimulator_v1.types.PreviewState):
            Output only. The state of the
            ``OrgPolicyViolationsPreview``.
        overlay (google.cloud.policysimulator_v1.types.OrgPolicyOverlay):
            Required. The proposed changes we are
            previewing violations for.
        violations_count (int):
            Output only. The number of [OrgPolicyViolations][] in this
            ``OrgPolicyViolationsPreview``. This count may differ from
            ``resource_summary.noncompliant_count`` because each
            [OrgPolicyViolation][google.cloud.policysimulator.v1.OrgPolicyViolation]
            is specific to a resource **and** constraint. If there are
            multiple constraints being evaluated (i.e. multiple policies
            in the overlay), a single resource may violate multiple
            constraints.
        resource_counts (google.cloud.policysimulator_v1.types.OrgPolicyViolationsPreview.ResourceCounts):
            Output only. A summary of the state of all
            resources scanned for compliance with the
            changed OrgPolicy.
        custom_constraints (MutableSequence[str]):
            Output only. The names of the constraints against which all
            ``OrgPolicyViolations`` were evaluated.

            If ``OrgPolicyOverlay`` only contains ``PolicyOverlay`` then
            it contains the name of the configured custom constraint,
            applicable to the specified policies. Otherwise it contains
            the name of the constraint specified in
            ``CustomConstraintOverlay``.

            Format:
            ``organizations/{organization_id}/customConstraints/{custom_constraint_id}``

            Example:
            ``organizations/123/customConstraints/custom.createOnlyE2TypeVms``
        create_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. Time when this ``OrgPolicyViolationsPreview``
            was created.
    """

    class ResourceCounts(proto.Message):
        r"""A summary of the state of all resources scanned for
        compliance with the changed OrgPolicy.

        Attributes:
            scanned (int):
                Output only. Number of resources checked for
                compliance.
                Must equal:  unenforced + noncompliant +
                compliant + error
            noncompliant (int):
                Output only. Number of scanned resources with
                at least one violation.
            compliant (int):
                Output only. Number of scanned resources with
                zero violations.
            unenforced (int):
                Output only. Number of resources where the constraint was
                not enforced, i.e. the Policy set ``enforced: false`` for
                that resource.
            errors (int):
                Output only. Number of resources that
                returned an error when scanned.
        """

        scanned: int = proto.Field(
            proto.INT32,
            number=1,
        )
        noncompliant: int = proto.Field(
            proto.INT32,
            number=2,
        )
        compliant: int = proto.Field(
            proto.INT32,
            number=3,
        )
        unenforced: int = proto.Field(
            proto.INT32,
            number=4,
        )
        errors: int = proto.Field(
            proto.INT32,
            number=5,
        )

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    state: "PreviewState" = proto.Field(
        proto.ENUM,
        number=2,
        enum="PreviewState",
    )
    overlay: "OrgPolicyOverlay" = proto.Field(
        proto.MESSAGE,
        number=3,
        message="OrgPolicyOverlay",
    )
    violations_count: int = proto.Field(
        proto.INT32,
        number=4,
    )
    resource_counts: ResourceCounts = proto.Field(
        proto.MESSAGE,
        number=5,
        message=ResourceCounts,
    )
    custom_constraints: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=6,
    )
    create_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=7,
        message=timestamp_pb2.Timestamp,
    )


class OrgPolicyViolation(proto.Message):
    r"""OrgPolicyViolation is a resource representing a single
    resource violating a single OrgPolicy constraint.

    Attributes:
        name (str):
            The name of the ``OrgPolicyViolation``. Example:
            organizations/my-example-org/locations/global/orgPolicyViolationsPreviews/506a5f7f/orgPolicyViolations/38ce\`
        resource (google.cloud.policysimulator_v1.types.ResourceContext):
            The resource violating the constraint.
        custom_constraint (google.cloud.orgpolicy_v2.types.CustomConstraint):
            The custom constraint being violated.
        error (google.rpc.status_pb2.Status):
            Any error encountered during the evaluation.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    resource: "ResourceContext" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="ResourceContext",
    )
    custom_constraint: constraint.CustomConstraint = proto.Field(
        proto.MESSAGE,
        number=3,
        message=constraint.CustomConstraint,
    )
    error: status_pb2.Status = proto.Field(
        proto.MESSAGE,
        number=4,
        message=status_pb2.Status,
    )


class ResourceContext(proto.Message):
    r"""ResourceContext provides the context we know about a
    resource. It is similar in concept to
    google.cloud.asset.v1.Resource, but focuses on the information
    specifically used by Simulator.

    Attributes:
        resource (str):
            The full name of the resource. Example:
            ``//compute.googleapis.com/projects/my_project_123/zones/zone1/instances/instance1``

            See `Resource
            names <https://cloud.google.com/apis/design/resource_names#full_resource_name>`__
            for more information.
        asset_type (str):
            The asset type of the resource as defined by CAIS.

            Example: ``compute.googleapis.com/Firewall``

            See `Supported asset
            types <https://cloud.google.com/asset-inventory/docs/supported-asset-types>`__
            for more information.
        ancestors (MutableSequence[str]):
            The ancestry path of the resource in Google Cloud `resource
            hierarchy <https://cloud.google.com/resource-manager/docs/cloud-platform-resource-hierarchy>`__,
            represented as a list of relative resource names. An
            ancestry path starts with the closest ancestor in the
            hierarchy and ends at root. If the resource is a project,
            folder, or organization, the ancestry path starts from the
            resource itself.

            Example:
            ``["projects/123456789", "folders/5432", "organizations/1234"]``
    """

    resource: str = proto.Field(
        proto.STRING,
        number=1,
    )
    asset_type: str = proto.Field(
        proto.STRING,
        number=2,
    )
    ancestors: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=3,
    )


class OrgPolicyOverlay(proto.Message):
    r"""The proposed changes to OrgPolicy.

    Attributes:
        policies (MutableSequence[google.cloud.policysimulator_v1.types.OrgPolicyOverlay.PolicyOverlay]):
            Optional. The OrgPolicy changes to preview
            violations for.
            Any existing OrgPolicies with the same name will
            be overridden in the simulation. That is,
            violations will be determined as if all policies
            in the overlay were created or updated.
        custom_constraints (MutableSequence[google.cloud.policysimulator_v1.types.OrgPolicyOverlay.CustomConstraintOverlay]):
            Optional. The OrgPolicy CustomConstraint changes to preview
            violations for.

            Any existing CustomConstraints with the same name will be
            overridden in the simulation. That is, violations will be
            determined as if all custom constraints in the overlay were
            instantiated.

            Only a single custom_constraint is supported in the overlay
            at a time. For evaluating multiple constraints, multiple
            ``GenerateOrgPolicyViolationsPreview`` requests are made,
            where each request evaluates a single constraint.
    """

    class PolicyOverlay(proto.Message):
        r"""A change to an OrgPolicy.

        Attributes:
            policy_parent (str):
                Optional. The parent of the policy we are
                attaching to. Example: "projects/123456".
            policy (google.cloud.orgpolicy_v2.types.Policy):
                Optional. The new or updated OrgPolicy.
        """

        policy_parent: str = proto.Field(
            proto.STRING,
            number=1,
        )
        policy: gcov_orgpolicy.Policy = proto.Field(
            proto.MESSAGE,
            number=2,
            message=gcov_orgpolicy.Policy,
        )

    class CustomConstraintOverlay(proto.Message):
        r"""A change to an OrgPolicy custom constraint.

        Attributes:
            custom_constraint_parent (str):
                Optional. Resource the constraint is attached
                to. Example: "organization/987654".
            custom_constraint (google.cloud.orgpolicy_v2.types.CustomConstraint):
                Optional. The new or updated custom
                constraint.
        """

        custom_constraint_parent: str = proto.Field(
            proto.STRING,
            number=1,
        )
        custom_constraint: constraint.CustomConstraint = proto.Field(
            proto.MESSAGE,
            number=2,
            message=constraint.CustomConstraint,
        )

    policies: MutableSequence[PolicyOverlay] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=PolicyOverlay,
    )
    custom_constraints: MutableSequence[CustomConstraintOverlay] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message=CustomConstraintOverlay,
    )


class CreateOrgPolicyViolationsPreviewOperationMetadata(proto.Message):
    r"""CreateOrgPolicyViolationsPreviewOperationMetadata is metadata
    about an OrgPolicyViolationsPreview generations operation.

    Attributes:
        request_time (google.protobuf.timestamp_pb2.Timestamp):
            Time when the request was received.
        start_time (google.protobuf.timestamp_pb2.Timestamp):
            Time when the request started processing,
            i.e., when the state was set to RUNNING.
        state (google.cloud.policysimulator_v1.types.PreviewState):
            Output only. The current state of the
            operation.
        resources_found (int):
            Total number of resources that need scanning. Should equal
            resource_scanned + resources_pending
        resources_scanned (int):
            Number of resources already scanned.
        resources_pending (int):
            Number of resources still to scan.
    """

    request_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=1,
        message=timestamp_pb2.Timestamp,
    )
    start_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=2,
        message=timestamp_pb2.Timestamp,
    )
    state: "PreviewState" = proto.Field(
        proto.ENUM,
        number=3,
        enum="PreviewState",
    )
    resources_found: int = proto.Field(
        proto.INT32,
        number=4,
    )
    resources_scanned: int = proto.Field(
        proto.INT32,
        number=5,
    )
    resources_pending: int = proto.Field(
        proto.INT32,
        number=6,
    )


class ListOrgPolicyViolationsPreviewsRequest(proto.Message):
    r"""ListOrgPolicyViolationsPreviewsRequest is the request message for
    [OrgPolicyViolationsPreviewService.ListOrgPolicyViolationsPreviews][google.cloud.policysimulator.v1.OrgPolicyViolationsPreviewService.ListOrgPolicyViolationsPreviews].

    Attributes:
        parent (str):
            Required. The parent the violations are scoped to. Format:
            ``organizations/{organization}/locations/{location}``

            Example: ``organizations/my-example-org/locations/global``
        page_size (int):
            Optional. The maximum number of items to
            return. The service may return fewer than this
            value. If unspecified, at most 5 items will be
            returned. The maximum value is 10; values above
            10 will be coerced to 10.
        page_token (str):
            Optional. A page token, received from a
            previous call. Provide this to retrieve the
            subsequent page.

            When paginating, all other parameters must match
            the call that provided the page token.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=2,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=3,
    )


class ListOrgPolicyViolationsPreviewsResponse(proto.Message):
    r"""ListOrgPolicyViolationsPreviewsResponse is the response message for
    [OrgPolicyViolationsPreviewService.ListOrgPolicyViolationsPreviews][google.cloud.policysimulator.v1.OrgPolicyViolationsPreviewService.ListOrgPolicyViolationsPreviews].

    Attributes:
        org_policy_violations_previews (MutableSequence[google.cloud.policysimulator_v1.types.OrgPolicyViolationsPreview]):
            The list of OrgPolicyViolationsPreview
        next_page_token (str):
            A token that you can use to retrieve the next
            page of results. If this field is omitted, there
            are no subsequent pages.
    """

    @property
    def raw_page(self):
        return self

    org_policy_violations_previews: MutableSequence["OrgPolicyViolationsPreview"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message="OrgPolicyViolationsPreview",
        )
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class GetOrgPolicyViolationsPreviewRequest(proto.Message):
    r"""GetOrgPolicyViolationsPreviewRequest is the request message for
    [OrgPolicyViolationsPreviewService.GetOrgPolicyViolationsPreview][google.cloud.policysimulator.v1.OrgPolicyViolationsPreviewService.GetOrgPolicyViolationsPreview].

    Attributes:
        name (str):
            Required. The name of the
            OrgPolicyViolationsPreview to get.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class CreateOrgPolicyViolationsPreviewRequest(proto.Message):
    r"""CreateOrgPolicyViolationsPreviewRequest is the request message for
    [OrgPolicyViolationsPreviewService.CreateOrgPolicyViolationsPreview][google.cloud.policysimulator.v1.OrgPolicyViolationsPreviewService.CreateOrgPolicyViolationsPreview].

    Attributes:
        parent (str):
            Required. The organization under which this
            [OrgPolicyViolationsPreview][google.cloud.policysimulator.v1.OrgPolicyViolationsPreview]
            will be created.

            Example: ``organizations/my-example-org/locations/global``
        org_policy_violations_preview (google.cloud.policysimulator_v1.types.OrgPolicyViolationsPreview):
            Required. The
            [OrgPolicyViolationsPreview][google.cloud.policysimulator.v1.OrgPolicyViolationsPreview]
            to generate.
        org_policy_violations_preview_id (str):
            Optional. An optional user-specified ID for the
            [OrgPolicyViolationsPreview][google.cloud.policysimulator.v1.OrgPolicyViolationsPreview].
            If not provided, a random ID will be generated.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    org_policy_violations_preview: "OrgPolicyViolationsPreview" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="OrgPolicyViolationsPreview",
    )
    org_policy_violations_preview_id: str = proto.Field(
        proto.STRING,
        number=3,
    )


class ListOrgPolicyViolationsRequest(proto.Message):
    r"""ListOrgPolicyViolationsRequest is the request message for
    [OrgPolicyViolationsPreviewService.ListOrgPolicyViolations][google.cloud.policysimulator.v1.OrgPolicyViolationsPreviewService.ListOrgPolicyViolations].

    Attributes:
        parent (str):
            Required. The OrgPolicyViolationsPreview to
            get OrgPolicyViolations from. Format:

            organizations/{organization}/locations/{location}/orgPolicyViolationsPreviews/{orgPolicyViolationsPreview}
        page_size (int):
            Optional. The maximum number of items to
            return. The service may return fewer than this
            value. If unspecified, at most 1000 items will
            be returned. The maximum value is 1000; values
            above 1000 will be coerced to 1000.
        page_token (str):
            Optional. A page token, received from a
            previous call. Provide this to retrieve the
            subsequent page.

            When paginating, all other parameters must match
            the call that provided the page token.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=2,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=3,
    )


class ListOrgPolicyViolationsResponse(proto.Message):
    r"""ListOrgPolicyViolationsResponse is the response message for
    [OrgPolicyViolationsPreviewService.ListOrgPolicyViolations][google.cloud.policysimulator.v1.OrgPolicyViolationsPreviewService.ListOrgPolicyViolations]

    Attributes:
        org_policy_violations (MutableSequence[google.cloud.policysimulator_v1.types.OrgPolicyViolation]):
            The list of OrgPolicyViolations
        next_page_token (str):
            A token that you can use to retrieve the next
            page of results. If this field is omitted, there
            are no subsequent pages.
    """

    @property
    def raw_page(self):
        return self

    org_policy_violations: MutableSequence["OrgPolicyViolation"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="OrgPolicyViolation",
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


__all__ = tuple(sorted(__protobuf__.manifest))
